@@ -1,14 +1,47 @@
 from hyperliquid.api import API
-from hyperliquid.utils.types import Any, Callable, Meta, SpotMeta, Optional, Subscription, cast, Cloid
+from hyperliquid.utils.types import (
+    Any,
+    Callable,
+    Cloid,
+    Meta,
+    Optional,
+    SpotMeta,
+    SpotMetaAndAssetCtxs,
+    Subscription,
+    cast,
+)
 from hyperliquid.websocket_manager import WebsocketManager
 
 
 class Info(API):
-    def __init__(self, base_url=None, skip_ws=False):
+    def __init__(
+        self,
+        base_url: Optional[str] = None,
+        skip_ws: Optional[bool] = False,
+        meta: Optional[Meta] = None,
+        spot_meta: Optional[SpotMeta] = None,
+    ):
         super().__init__(base_url)
         if not skip_ws:
             self.ws_manager = WebsocketManager(self.base_url)
             self.ws_manager.start()
+        if meta is None:
+            meta = self.meta()
+
+        if spot_meta is None:
+            spot_meta = self.spot_meta()
+
+        self.coin_to_asset = {asset_info["name"]: asset for (asset, asset_info) in enumerate(meta["universe"])}
+        self.name_to_coin = {asset_info["name"]: asset_info["name"] for asset_info in meta["universe"]}
+
+        # spot assets start at 10000
+        for spot_info in spot_meta["universe"]:
+            self.coin_to_asset[spot_info["name"]] = spot_info["index"] + 10000
+            self.name_to_coin[spot_info["name"]] = spot_info["name"]
+            base, quote = spot_info["tokens"]
+            name = f'{spot_meta["tokens"][base]["name"]}/{spot_meta["tokens"][quote]["name"]}'
+            if name not in self.name_to_coin:
+                self.name_to_coin[name] = spot_info["name"]
 
     def user_state(self, address: str) -> Any:
         """Retrieve trading details about a user.
@@ -172,6 +205,41 @@ class Info(API):
         """
         return cast(Meta, self.post("/info", {"type": "meta"}))
 
+    def meta_and_asset_ctxs(self) -> Any:
+        """Retrieve exchange MetaAndAssetCtxs
+
+        POST /info
+
+        Returns:
+            [
+                {
+                    universe: [
+                        {
+                            'name': str,
+                            'szDecimals': int
+                            'maxLeverage': int,
+                            'onlyIsolated': bool,
+                        },
+                        ...
+                    ]
+                },
+            [
+                {
+                    "dayNtlVlm": float string,
+                    "funding": float string,
+                    "impactPxs": Optional([float string, float string]),
+                    "markPx": Optional(float string),
+                    "midPx": Optional(float string),
+                    "openInterest": float string,
+                    "oraclePx": float string,
+                    "premium": Optional(float string),
+                    "prevDayPx": float string
+                },
+                ...
+            ]
+        """
+        return self.post("/info", {"type": "metaAndAssetCtxs"})
+
     def spot_meta(self) -> SpotMeta:
         """Retrieve exchange spot metadata
 
@@ -179,18 +247,23 @@ class Info(API):
 
         Returns:
             {
+                universe: [
+                    {
+                        tokens: [int, int],
+                        name: str,
+                        index: int,
+                        isCanonical: bool
+                    },
+                    ...
+                ],
                 tokens: [
                     {
                         name: str,
                         szDecimals: int,
-                        weiDecimals: int
-                    },
-                    ...
-                ],
-                universe: [
-                    {
-                        name: str,
-                        tokens: [int, int]
+                        weiDecimals: int,
+                        index: int,
+                        tokenId: str,
+                        isCanonical: bool
                     },
                     ...
                 ]
@@ -198,13 +271,55 @@ class Info(API):
         """
         return cast(SpotMeta, self.post("/info", {"type": "spotMeta"}))
 
-    def funding_history(self, coin: str, startTime: int, endTime: Optional[int] = None) -> Any:
+    def spot_meta_and_asset_ctxs(self) -> SpotMetaAndAssetCtxs:
+        """Retrieve exchange spot asset contexts
+        POST /info
+        Returns:
+            [
+                {
+                    universe: [
+                        {
+                            tokens: [int, int],
+                            name: str,
+                            index: int,
+                            isCanonical: bool
+                        },
+                        ...
+                    ],
+                    tokens: [
+                        {
+                            name: str,
+                            szDecimals: int,
+                            weiDecimals: int,
+                            index: int,
+                            tokenId: str,
+                            isCanonical: bool
+                        },
+                        ...
+                    ]
+                },
+                [
+                    {
+                        dayNtlVlm: float string,
+                        markPx: float string,
+                        midPx: Optional(float string),
+                        prevDayPx: float string,
+                        circulatingSupply: float string,
+                        coin: str
+                    }
+                    ...
+                ]
+            ]
+        """
+        return cast(SpotMetaAndAssetCtxs, self.post("/info", {"type": "spotMetaAndAssetCtxs"}))
+
+    def funding_history(self, name: str, startTime: int, endTime: Optional[int] = None) -> Any:
         """Retrieve funding history for a given coin
 
         POST /info
 
         Args:
-            coin (str): Coin to retrieve funding history for.
+            name (str): Coin to retrieve funding history for.
             startTime (int): Unix timestamp in milliseconds.
             endTime (int): Unix timestamp in milliseconds.
 
@@ -219,6 +334,7 @@ class Info(API):
                 ...
             ]
         """
+        coin = self.name_to_coin[name]
         if endTime is not None:
             return self.post(
                 "/info", {"type": "fundingHistory", "coin": coin, "startTime": startTime, "endTime": endTime}
@@ -243,13 +359,13 @@ class Info(API):
             return self.post("/info", {"type": "userFunding", "user": user, "startTime": startTime, "endTime": endTime})
         return self.post("/info", {"type": "userFunding", "user": user, "startTime": startTime})
 
-    def l2_snapshot(self, coin: str) -> Any:
+    def l2_snapshot(self, name: str) -> Any:
         """Retrieve L2 snapshot for a given coin
 
         POST /info
 
         Args:
-            coin (str): Coin to retrieve L2 snapshot for.
+            name (str): Coin to retrieve L2 snapshot for.
 
         Returns:
             {
@@ -268,15 +384,15 @@ class Info(API):
                 time: int
             }
         """
-        return self.post("/info", {"type": "l2Book", "coin": coin})
+        return self.post("/info", {"type": "l2Book", "coin": self.name_to_coin[name]})
 
-    def candles_snapshot(self, coin: str, interval: str, startTime: int, endTime: int) -> Any:
+    def candles_snapshot(self, name: str, interval: str, startTime: int, endTime: int) -> Any:
         """Retrieve candles snapshot for a given coin
 
         POST /info
 
         Args:
-            coin (str): Coin to retrieve candles snapshot for.
+            name (str): Coin to retrieve candles snapshot for.
             interval (str): Candlestick interval.
             startTime (int): Unix timestamp in milliseconds.
             endTime (int): Unix timestamp in milliseconds.
@@ -298,7 +414,7 @@ class Info(API):
                 ...
             ]
         """
-        req = {"coin": coin, "interval": interval, "startTime": startTime, "endTime": endTime}
+        req = {"coin": self.name_to_coin[name], "interval": interval, "startTime": startTime, "endTime": endTime}
         return self.post("/info", {"type": "candleSnapshot", "req": req})
 
     def user_fees(self, address: str) -> Any:
@@ -356,14 +472,27 @@ class Info(API):
     def query_sub_accounts(self, user: str) -> Any:
         return self.post("/info", {"type": "subAccounts", "user": user})
 
+    def query_user_to_multi_sig_signers(self, multi_sig_user: str) -> Any:
+        return self.post("/info", {"type": "userToMultiSigSigners", "user": multi_sig_user})
+
+    def query_signer_to_multi_sig_user(self, signer: str) -> Any:
+        return self.post("/info", {"type": "signerToMultiSigUser", "signer": signer})
+
     def subscribe(self, subscription: Subscription, callback: Callable[[Any], None]) -> int:
+        if subscription["type"] == "l2Book" or subscription["type"] == "trades" or subscription["type"] == "candle":
+            subscription["coin"] = self.name_to_coin[subscription["coin"]]
         if self.ws_manager is None:
             raise RuntimeError("Cannot call subscribe since skip_ws was used")
         else:
             return self.ws_manager.subscribe(subscription, callback)
 
     def unsubscribe(self, subscription: Subscription, subscription_id: int) -> bool:
+        if subscription["type"] == "l2Book" or subscription["type"] == "trades" or subscription["type"] == "candle":
+            subscription["coin"] = self.name_to_coin[subscription["coin"]]
         if self.ws_manager is None:
             raise RuntimeError("Cannot call unsubscribe since skip_ws was used")
         else:
             return self.ws_manager.unsubscribe(subscription, subscription_id)
+
+    def name_to_asset(self, name: str) -> int:
+        return self.coin_to_asset[self.name_to_coin[name]]
